############################################################
#                                                          #
#       AUTHOR     : F. BELLACHIA                          #
#       LABORATORY : L.A.P.P.                              #
#       VERSION    : v4r0p1                                #
#       DATE       : 30/05/02                              #
#                                                          #
############################################################
# Copyright (c) 2002 LAPP Annecy, UMR-IN2P3-CNRS (France). #
#                                                          #
# Redistribution and use in source and binary forms, with  #
# or without modification, are permitted provided that the #
# following conditions are met:                            #
# 1. Redistributions of source code must retain the above  #
#    copyright notice, this list of conditions and the     #
#    following disclaimer.                                 #
# 2. Redistributions in binary form must reproduce the     #
#    above copyright notice, this list of conditions and   #
#    the following disclaimer in the documentation andor   #
#    other materials provided with the distribution.       #
# 3. All advertising materials mentioning features or use  #
#    of this software must display the following           #
#    acknowledgement:                                      #
#      This product includes software developed by the     #
#      Computer Application Development Group at LAPP      #
#      Annecy  (Laboratoire d'Annecy le vieux de Physique  #
#      des Particules - UMR-IN2P3-CNRS).                   #
# 4. Neither the name of the Institute nor of the          #
#    Laboratory may be used to endorse or promote products #
#    derived from this software without specific prior     #
#    written permission.                                   #
#                                                          #
# THIS SOFTWARE IS PROVIDED BY THE LAPP AND CONTRIBUTORS   #
# ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,         #
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF #
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE #
# DISCLAIMED. IN NO EVENT SHALL THE LAPP OR CONTRIBUTORS   #
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, #
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT  #
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; #
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) #
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER   #
# IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING        #
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE   #
# USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY #
# OF SUCH DAMAGE.                                          #
############################################################

#-------------------------------------------------------------------------------
#- record function -------------------------------------------------------------
#-------------------------------------------------------------------------------
#
#  This function is used to store name of signal and its value into the array
#-------------------------------------------------------------------------------
function record(signame, value) {

  if(sprintf("%d", signame) == 0){
    signo = sprintf("%d", value);

    if(signo != 0) {
      nsig++;
      siglist[nsig] = sprintf("\"%s\", %2d", signame, signo);
    }
    else {
      for (n = 1; n <= nsig; n++) {
        split(siglist[n], tab, ",");
        name = sprintf("\"%s\"", value);

        if (tab[1] == name) {
          nsig++;
          siglist[nsig] = sprintf("\"%s\", %2d", signame, tab[2]);
        }
      }
    }
  }
}

#-------------------------------------------------------------------------------
#- epilogue processing function ------------------------------------------------
#-------------------------------------------------------------------------------
BEGIN {
  output = ARGV[1];
  nsig   = 0;
  signo  = 0;

# FCA, modified the definition of module on Linux to make script working for GNU Awk 3.1.3 (Linux-x86_64)
# --------------------------------------------------------------------------------- See SPR #

  "uname -s" | getline os;
  "uname -r" | getline revision;
  "date +\"%d %b %Y\"" | getline date;

  if (os == "Linux") {
    command = "xargs basename"
    print output |& command;
    close(command,"to");
    command |& getline module;
    close(command);
    ARGV[1] = "/usr/include/bits/signum.h";
  }
  else {
    "basename "ARGV[1] | getline module;
    ARGV[1] = "/usr/include/signal.h";
  }


  printf("/*****************************************/\n")    > output
  printf("/* MODULE: %-29s */\n", module)                   >> output
  printf("/* USER  : %-29s */\n", ENVIRON["USER"])          >> output
  printf("/* DATE  : %-29s */\n", date)                     >> output
  printf("/* SYSTEM: %-29s */\n", os)                       >> output
  printf("/* OS rev: %-29s */\n", revision)                 >> output
  printf("/*                                       */\n")   >> output
  printf("/* This file was automatically generated */\n")   >> output
  printf("/* from %-32s */\n", ARGV[1])                     >> output
  printf("/* by the awk script \"CbfSigConv.awk\".   */\n") >> output
  printf("/*****************************************/\n\n") >> output

  sub(".h", "_h", module);

  printf("#ifndef %s\n", module) >> output
  printf("#define %s\n\n", module) >> output

  printf("extern void psignal(int, const char *);\n\n") >> output
  printf("struct sigdesc {\n")                          >> output
  printf("  char *name;\n")                             >> output
  printf("  int  number;\n")                            >> output
  printf("};\n\n")                                      >> output
  printf("struct sigdesc CbfSigDesc[] = {\n")           >> output
}

#-------------------------------------------------------------------------------
#- pattern processing rules ----------------------------------------------------
#-------------------------------------------------------------------------------
#
#  Try to find:
#     -> #define SIG<xx> <num> or SIG<yy>
#     -> #  define SIG<xx> <num> or SIG<yy>
#     -> #define _SIG<xx> <num> or _SIG<yy>
#-------------------------------------------------------------------------------
/^#define[ \t][ \t]*SIG[A-Z]/ { record($2, $3); }

/^#[ \t]+define[ \t][ \t]*SIG[A-Z]/ { record($3, $4); }

/^#[ \t]*define[ \t][ \t]*_SIG[A-Z]/ { record($3, $4); }

#-------------------------------------------------------------------------------
#- prologue processing function ------------------------------------------------
#-------------------------------------------------------------------------------
END {
  for (n = 1; n <= nsig; n++)
    if (siglist[n] != "")
      printf("  {%s},\n", siglist[n]) >> output

  printf("  {NULL, 0}\n};\n")       >> output
  printf("\n#endif /* %s */\n", module) >> output
}
